@@ -3787,4 +3787,6 @@ xmlCatalogGetPublic(const xmlChar *pubID) {
     return(NULL);
 }
 
+#define bottom_catalog
+#include "elfgcchack.h"
 #endif /* LIBXML_CATALOG_ENABLED */
